@@ -1,5 +1,6 @@
 // -*- C++ -*-
-/* Copyright (C) 1989, 1990, 1991, 1992 Free Software Foundation, Inc.
+/* Copyright (C) 1989, 1990, 1991, 1992, 2000, 2001
+   Free Software Foundation, Inc.
      Written by James Clark (jjc@jclark.com)
 
 This file is part of groff.
@@ -168,7 +169,7 @@ public:
   void begin_page(int);
   void end_page(int);
   void set_char(int, font *, const environment *, int w, const char *name);
-  void special(char *arg, const environment *env);
+  void special(char *arg, const environment *env, char type);
   void end_of_line();
   void draw(int code, int *p, int np, const environment *env);
 };
@@ -492,8 +493,10 @@ void dvi_printer::do_special(const char *s)
     out1(*s++);
 }
 
-void dvi_printer::special(char *arg, const environment *env)
+void dvi_printer::special(char *arg, const environment *env, char type)
 {
+  if (type != 'p')
+    return;
   moveto(env->hpos, env->vpos);
   do_special(arg);
 }
@@ -843,7 +846,7 @@ printer *make_printer()
     return new dvi_printer;
 }
 
-static void usage();
+static void usage(FILE *stream);
 
 int main(int argc, char **argv)
 {
@@ -851,13 +854,18 @@ int main(int argc, char **argv)
   static char stderr_buf[BUFSIZ];
   setbuf(stderr, stderr_buf);
   int c;
-  while ((c = getopt(argc, argv, "F:vw:d")) != EOF)
+  static const struct option long_options[] = {
+    { "help", no_argument, 0, CHAR_MAX + 1 },
+    { "version", no_argument, 0, 'v' },
+    { NULL, 0, 0, 0 }
+  };
+  while ((c = getopt_long(argc, argv, "F:vw:d", long_options, NULL)) != EOF)
     switch(c) {
     case 'v':
       {
 	extern const char *Version_string;
-	fprintf(stderr, "grodvi version %s\n", Version_string);
-	fflush(stderr);
+	printf("GNU grodvi (groff) version %s\n", Version_string);
+	exit(0);
 	break;
       }
     case 'w':
@@ -873,8 +881,13 @@ int main(int argc, char **argv)
     case 'F':
       font::command_line_font_dir(optarg);
       break;
+    case CHAR_MAX + 1: // --help
+      usage(stdout);
+      exit(0);
+      break;
     case '?':
-      usage();
+      usage(stderr);
+      exit(1);
       break;
     default:
       assert(0);
@@ -892,9 +905,8 @@ int main(int argc, char **argv)
   return 0;
 }
 
-static void usage()
+static void usage(FILE *stream)
 {
-  fprintf(stderr, "usage: %s [-dv] [-F dir] [-w n] [files ...]\n",
+  fprintf(stream, "usage: %s [-dv] [-F dir] [-w n] [files ...]\n",
 	  program_name);
-  exit(1);
 }
