@@ -1,5 +1,6 @@
 // -*- C++ -*-
-/* Copyright (C) 1989, 1990, 1991, 1992 Free Software Foundation, Inc.
+/* Copyright (C) 1989, 1990, 1991, 1992, 2000, 2001
+   Free Software Foundation, Inc.
      Written by James Clark (jjc@jclark.com)
 
 This file is part of groff.
@@ -26,6 +27,12 @@ Foundation, 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA. */
 #include "ps.h"
 #include <time.h>
 
+#ifdef NEED_DECLARATION_PUTENV
+extern "C" {
+  int putenv(const char *);
+}
+#endif /* NEED_DECLARATION_PUTENV */
+
 static int landscape_flag = 0;
 static int manual_feed_flag = 0;
 static int ncopies = 1;
@@ -507,7 +514,7 @@ public:
   void draw(int code, int *p, int np, const environment *env);
   void begin_page(int);
   void end_page(int);
-  void special(char *arg, const environment *env);
+  void special(char *arg, const environment *env, char type);
   font *make_font(const char *);
   void end_of_line();
 };
@@ -1204,8 +1211,10 @@ ps_printer::~ps_printer()
   fclose(tempfp);
 }
 
-void ps_printer::special(char *arg, const environment *env)
+void ps_printer::special(char *arg, const environment *env, char type)
 {
+  if (type != 'p')
+    return;
   typedef void (ps_printer::*SPECIAL_PROCP)(char *, const environment *);
   static struct {
     const char *name;
@@ -1467,21 +1476,28 @@ printer *make_printer()
   return new ps_printer;
 }
 
-static void usage();
+static void usage(FILE *stream);
 
 int main(int argc, char **argv)
 {
   program_name = argv[0];
+  string env;
   static char stderr_buf[BUFSIZ];
   setbuf(stderr, stderr_buf);
   int c;
-  while ((c = getopt(argc, argv, "F:glmc:w:vb:")) != EOF)
+  static const struct option long_options[] = {
+    { "help", no_argument, 0, CHAR_MAX + 1 },
+    { "version", no_argument, 0, 'v' },
+    { NULL, 0, 0, 0 }
+  };
+  while ((c = getopt_long(argc, argv, "F:P:glmc:w:vb:", long_options, NULL))
+	 != EOF)
     switch(c) {
     case 'v':
       {
 	extern const char *Version_string;
-	fprintf(stderr, "grops version %s\n", Version_string);
-	fflush(stderr);
+	printf("GNU grops (groff) version %s\n", Version_string);
+	exit(0);
 	break;
       }
     case 'c':
@@ -1502,6 +1518,14 @@ int main(int argc, char **argv)
     case 'F':
       font::command_line_font_dir(optarg);
       break;
+    case 'P':
+      env = "GROPS_PROLOGUE";
+      env += '=';
+      env += optarg;
+      env += '\0';
+      if (putenv(strsave(env.contents())))
+	fatal("putenv failed");
+      break;
     case 'w':
       if (sscanf(optarg, "%d", &linewidth) != 1 || linewidth < 0) {
 	error("bad linewidth `%1'", optarg);
@@ -1513,8 +1537,13 @@ int main(int argc, char **argv)
       broken_flags = atoi(optarg);
       bflag = 1;
       break;
+    case CHAR_MAX + 1: // --help
+      usage(stdout);
+      exit(0);
+      break;
     case '?':
-      usage();
+      usage(stderr);
+      exit(1);
       break;
     default:
       assert(0);
@@ -1533,9 +1562,9 @@ int main(int argc, char **argv)
   return 0;
 }
 
-static void usage()
+static void usage(FILE *stream)
 {
-  fprintf(stderr, "usage: %s [-glmv] [-b n] [-c n] [-w n] [-F dir] [files ...]\n",
-	  program_name);
-  exit(1);
+  fprintf(stream,
+    "usage: %s [-glmv] [-b n] [-c n] [-w n] [-P prologue] [-F dir] [files ...]\n",
+    program_name);
 }
