@@ -1,5 +1,5 @@
 // -*- C++ -*-
-/* Copyright (C) 1989, 1990, 1991, 1992 Free Software Foundation, Inc.
+/* Copyright (C) 1989, 1990, 1991, 1992, 2000 Free Software Foundation, Inc.
      Written by James Clark (jjc@jclark.com)
 
 This file is part of groff.
@@ -24,7 +24,13 @@ Foundation, 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA. */
 
 #include "ps.h"
 
-#define PROLOGUE "prologue"
+#ifdef NEED_DECLARATION_PUTENV
+extern "C" {
+  int putenv(const char *);
+}
+#endif /* NEED_DECLARATION_PUTENV */
+
+#define GROPS_PROLOGUE "prologue"
 
 static void print_ps_string(const string &s, FILE *outfp);
 
@@ -259,9 +265,18 @@ void resource_manager::output_prolog(ps_output &out)
   FILE *outfp = out.get_file();
   out.end_line();
   char *path;
-  FILE *fp = font::open_file(PROLOGUE, &path);
+  if (!getenv("GROPS_PROLOGUE")) {
+    string e = "GROPS_PROLOGUE";
+    e += '=';
+    e += GROPS_PROLOGUE;
+    e += '\0';
+    if (putenv(strsave(e.contents())))
+      fatal("putenv failed");
+  }
+  char *prologue = getenv("GROPS_PROLOGUE");
+  FILE *fp = font::open_file(prologue, &path);
   if (!fp)
-    fatal("can't find `%1'", PROLOGUE);
+    fatal("can't find `%1'", prologue);
   fputs("%%BeginResource: ", outfp);
   procset_resource->print_type_and_name(outfp);
   putc('\n', outfp);
