@@ -26,12 +26,20 @@ extern "C" {
   const char *if_to_a(int, int);
 }
 
+/* stdio.h on IRIX and OSF/1 include getopt.h */
+
+#if !(defined(__sgi) || (defined(__osf__) && defined(__alpha)))
 #include <groff-getopt.h>
+#endif
 
 char *strsave(const char *s);
 int is_prime(unsigned);
 
 #include <stdio.h>
+#include <string.h>
+#ifdef HAVE_STRINGS_H
+#include <strings.h>
+#endif
 
 FILE *xtmpfile(char **namep=0, char *postfix=0, int do_unlink=1);
 char *xtmptemplate(char *extension=0);
@@ -57,6 +65,17 @@ inline int illegal_input_char(int c)
   return c >= 0 && illegal_char_table[c];
 }
 
+#if !defined(_AIX) && !defined(sinix) && !defined(__sinix__)
+#ifdef HAVE_STRNCASECMP
+#ifdef NEED_DECLARATION_STRNCASECMP
+extern "C" {
+  // SunOS's string.h fails to declare this.
+  int strncasecmp(const char *, const char *, int);
+}
+#endif /* NEED_DECLARATION_STRNCASECMP */
+#endif /* HAVE_STRNCASECMP */
+#endif /* !_AIX && !sinix && !__sinix__ */
+
 #ifndef HAVE_STRCASECMP
 #define strcasecmp(a,b) strcmp((a),(b))
 #endif
