@@ -1,5 +1,5 @@
 // -*- C++ -*-
-/* Copyright (C) 1989, 1990, 1991, 1992 Free Software Foundation, Inc.
+/* Copyright (C) 1989, 1990, 1991, 1992, 2000 Free Software Foundation, Inc.
      Written by James Clark (jjc@jclark.com)
 
 This file is part of groff.
@@ -58,7 +58,7 @@ struct font_widths_cache {
   int point_size;
   int *width;
 
-  font_widths_cache(int, int, font_widths_cache *);
+  font_widths_cache(int, int, font_widths_cache * = 0);
   ~font_widths_cache();
 };
 
@@ -222,7 +222,7 @@ int font::is_special()
 }
 
 font_widths_cache::font_widths_cache(int ps, int ch_size,
-				     font_widths_cache *p = 0)
+				     font_widths_cache *p)
 : next(p), point_size(ps)
 {
   width = new int[ch_size];
