@@ -85,7 +85,7 @@
 /* database imports */
 
 extern void HGPrintElt(ELT *element, int baseline);
-extern ELT *DBInit(void);
+extern ELT *DBInit();
 extern ELT *DBRead(register FILE *file);
 extern POINT *PTInit();
 extern POINT *PTMakePoint(float x, float y, POINT **pplist);
@@ -211,21 +211,20 @@ int SUNFILE = FALSE;		/* TRUE if SUN gremlin file */
 int compatibility_flag = FALSE;	/* TRUE if in compatibility mode */
 
 
-void getres(void);
+void getres();
 char *doinput(FILE *fp);
 void conv(register FILE *fp, int baseline);
-void savestate(void);
+void savestate();
 int has_polygon(register ELT *elist);
 void interpret(char *line);
 
 
 void
-usage()
+usage(FILE *stream)
 {
-  fprintf(stderr,
+  fprintf(stream,
 	  "usage: %s [ -vCs ] [ -M dir ] [ -F dir ] [ -T dev ] [ file ]\n",
 	  program_name);
-  exit(1);
 }
 
 
@@ -260,13 +259,6 @@ main(int argc,
 	file[gfil++] = NULL;
 	break;
 
-      case 'v':
-      {
-	extern const char *Version_string;
-	fprintf(stderr, "GNU grn version %s\n", Version_string);
-	fflush(stderr);
-	break;
-      }
       case 'C':		/* compatibility mode */
 	compatibility_flag = TRUE;
 	break;
@@ -287,13 +279,25 @@ main(int argc,
 	sflag = 1;
 	break;
 
+      case '-':
+	if (strcmp(*argv,"--version")==0) {
+      case 'v':
+	  extern const char *Version_string;
+	  printf("GNU grn (groff) version %s\n", Version_string);
+	  exit(0);
+	  break;
+	}
+	if (strcmp(*argv,"--help")==0) {
       case '?':
-	usage();
-	break;
-
+	  usage(stdout);
+	  exit(0);
+	  break;
+	}
+	// fallthrough
       default:
 	error("unknown switch: %1", c);
-	usage();
+	usage(stderr);
+	exit(1);
       }
   }
 
@@ -357,7 +361,7 @@ operand(int *argcp,
  *----------------------------------------------------------------------------*/
 
 void
-getres(void)
+getres()
 {
   int linepiece;
 
@@ -416,7 +420,7 @@ doinput(FILE *fp)
  *----------------------------------------------------------------------------*/
 
 void
-initpic(void)
+initpic()
 {
   register int i;
 
@@ -650,7 +654,7 @@ conv(register FILE *fp,
  *----------------------------------------------------------------------------*/
 
 void
-savestate(void)
+savestate()
 {
   register int i;
 
