@@ -1,5 +1,6 @@
 // -*- C++ -*-
-/* Copyright (C) 1989, 1990, 1991, 1992 Free Software Foundation, Inc.
+/* Copyright (C) 1989, 1990, 1991, 1992, 2000, 2001
+   Free Software Foundation, Inc.
      Written by James Clark (jjc@jclark.com)
 
 This file is part of groff.
@@ -19,6 +20,7 @@ with groff; see the file COPYING.  If not, write to the Free Software
 Foundation, 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA. */
 
 #include "table.h"
+#include "htmlindicate.h"
 
 #define MAX_POINT_SIZE 99
 #define MAX_VERTICAL_SPACING 72
@@ -217,6 +219,8 @@ void process_input_file(FILE *fp)
       break;
     case HAD_TS:
       if (c == ' ' || c == '\n' || compatible_flag) {
+	printf(".if '\\*(.T'html' \\X(table-start(\n");
+	html_begin_suppress(0);
 	putchar('.');
 	putchar('T');
 	putchar('S');
@@ -229,24 +233,25 @@ void process_input_file(FILE *fp)
 	  c = getc(fp);
 	}
 	putchar('\n');
-	printf(".if '\\*(.T'html' \\X(table-start(\n");
 	current_lineno++;
 	{
 	  table_input input(fp);
 	  process_table(input);
-	  if (input.ended())
-	    printf(".if '\\*(.T'html' \\X(table-end(\n");
 	  set_troff_location(current_filename, current_lineno);
 	  if (input.ended()) {
 	    fputs(".TE", stdout);
 	    while ((c = getc(fp)) != '\n') {
 	      if (c == EOF) {
+		printf(".if '\\*(.T'html' \\X(table-end(\n");
+		html_end_suppress(0);
 		putchar('\n');
 		return;
 	      }
 	      putchar(c);
 	    }
 	    putchar('\n');
+	    printf(".if '\\*(.T'html' \\X(table-end(\n");
+	    html_end_suppress(0);
 	    current_lineno++;
 	  }
 	}
@@ -1442,10 +1447,9 @@ void process_table(table_input &in)
     error("premature end of file");
 }
 
-static void usage()
+static void usage(FILE *stream)
 {
-  fprintf(stderr, "usage: %s [ -vC ] [ files... ]\n", program_name);
-  exit(1);
+  fprintf(stream, "usage: %s [ -vC ] [ files... ]\n", program_name);
 }
 
 int main(int argc, char **argv)
@@ -1454,7 +1458,12 @@ int main(int argc, char **argv)
   static char stderr_buf[BUFSIZ];
   setbuf(stderr, stderr_buf);
   int opt;
-  while ((opt = getopt(argc, argv, "vCT:")) != EOF)
+  static const struct option long_options[] = {
+    { "help", no_argument, 0, CHAR_MAX + 1 },
+    { "version", no_argument, 0, 'v' },
+    { NULL, 0, 0, 0 }
+  };
+  while ((opt = getopt_long(argc, argv, "vCT:", long_options, NULL)) != EOF)
     switch (opt) {
     case 'C':
       compatible_flag = 1;
@@ -1462,15 +1471,20 @@ int main(int argc, char **argv)
     case 'v':
       {
 	extern const char *Version_string;
-	fprintf(stderr, "GNU tbl version %s\n", Version_string);
-	fflush(stderr);
+	printf("GNU tbl (groff) version %s\n", Version_string);
+	exit(0);
 	break;
       }
     case 'T':
       // I'm sick of getting bug reports from IRIX users
       break;
+    case CHAR_MAX + 1: // --help
+      usage(stdout);
+      exit(0);
+      break;
     case '?':
-      usage();
+      usage(stderr);
+      exit(1);
       break;
     default:
       assert(0);
