@@ -1,5 +1,5 @@
 // -*- C++ -*-
-/* Copyright (C) 1989-2000 Free Software Foundation, Inc.
+/* Copyright (C) 1989-2000, 2001 Free Software Foundation, Inc.
      Written by James Clark (jjc@jclark.com)
 
 This file is part of groff.
@@ -84,18 +84,19 @@ public:
 
 int lflag = 0;
 char *spooler = 0;
-char *driver = 0;
+char *postdriver = 0;
+char *predriver = 0;
 
 possible_command commands[NCOMMANDS];
 
-int run_commands();
+int run_commands(int no_pipe);
 void print_commands();
 void append_arg_to_string(const char *arg, string &str);
 void handle_unknown_desc_command(const char *command, const char *arg,
 				 const char *filename, int lineno);
 const char *xbasename(const char *);
 
-void usage();
+void usage(FILE *stream);
 void help();
 
 int main(int argc, char **argv)
@@ -105,6 +106,7 @@ int main(int argc, char **argv)
   setbuf(stderr, stderr_buf);
   assert(NCOMMANDS <= MAX_COMMANDS);
   string Pargs, Largs, Fargs;
+  int vflag = 0;
   int Vflag = 0;
   int zflag = 0;
   int iflag = 0;
@@ -115,8 +117,14 @@ int main(int argc, char **argv)
   if (!command_prefix)
     command_prefix = PROG_PREFIX;
   commands[TROFF_INDEX].set_name(command_prefix, "troff");
-  while ((opt = getopt(argc, argv,
-		       "abCd:eEf:F:gGhiI:lL:m:M:n:No:pP:r:RsStT:UvVw:W:XzZ"))
+  static const struct option long_options[] = {
+    { "help", no_argument, 0, 'h' },
+    { "version", no_argument, 0, 'v' },
+    { NULL, 0, 0, 0 }
+  };
+  while ((opt = getopt_long(argc, argv,
+			    "abCd:eEf:F:gGhiI:lL:m:M:n:No:pP:r:RsStT:UvVw:W:XzZ",
+			    long_options, NULL))
 	 != EOF) {
     char buf[3];
     buf[0] = '-';
@@ -165,8 +173,23 @@ int main(int argc, char **argv)
       Vflag++;
       break;
     case 'v':
+      vflag = 1;
+      {
+	extern const char *Version_string;
+	printf("GNU groff version %s\n", Version_string);
+	printf("Copyright (C) 1989-2001 Free Software Foundation, Inc.\n"
+	       "GNU groff comes with ABSOLUTELY NO WARRANTY.\n"
+	       "You may redistribute copies of groff and its subprograms\n"
+	       "under the terms of the GNU General Public License.\n"
+	       "For more information about these matters, see the file named COPYING.\n");
+	printf("\ncalled subprograms:\n\n");
+        fflush(stdout);
+      }
+      commands[POST_INDEX].append_arg(buf);
+      // fall through
     case 'C':
       commands[SOELIM_INDEX].append_arg(buf);
+      commands[REFER_INDEX].append_arg(buf);
       commands[PIC_INDEX].append_arg(buf);
       commands[GRAP_INDEX].append_arg(buf);
       commands[TBL_INDEX].append_arg(buf);
@@ -191,6 +214,10 @@ int main(int argc, char **argv)
       safer_flag = 0;
       break;
     case 'T':
+      if (strcmp(optarg, "html") == 0) {
+	// force soelim to aid the html preprocessor
+	commands[SOELIM_INDEX].set_name(command_prefix, "soelim");
+      }
       if (strcmp(optarg, "Xps") == 0) {
 	warning("-TXps option is obsolete: use -X -Tps instead");
 	device = "ps";
@@ -235,33 +262,45 @@ int main(int argc, char **argv)
       Xflag++;
       break;
     case '?':
-      usage();
+      usage(stderr);
+      exit(1);
       break;
     default:
       assert(0);
       break;
     }
   }
-  if (safer_flag) {
+  if (safer_flag)
     commands[PIC_INDEX].append_arg("-S");
-    commands[TROFF_INDEX].insert_arg("-msafer");
-  } else {
+  else
     commands[TROFF_INDEX].insert_arg("-U");
-  }
   font::set_unknown_desc_command_handler(handle_unknown_desc_command);
   if (!font::load_desc())
     fatal("invalid device `%1'", device);
-  if (!driver)
+  if (!postdriver)
     fatal("no `postpro' command in DESC file for device `%1'", device);
+
+  if (predriver) {
+    commands[TROFF_INDEX].insert_arg(commands[TROFF_INDEX].get_name());
+    const char *p = Pargs.contents();
+    const char *end = p + Pargs.length();
+    while (p < end) {
+      // pass the device arguments to the predrivers as well
+      commands[TROFF_INDEX].insert_arg(p);
+      p = strchr(p, '\0') + 1;
+    }
+    commands[TROFF_INDEX].set_name(predriver);
+  }
+
   const char *real_driver = 0;
   if (Xflag) {
-    real_driver = driver;
-    driver = GXDITVIEW;
+    real_driver = postdriver;
+    postdriver = GXDITVIEW;
     commands[TROFF_INDEX].append_arg("-r" XREG "=", "1");
   }
-  if (driver)
-    commands[POST_INDEX].set_name(driver);
-  int gxditview_flag = driver && strcmp(xbasename(driver), GXDITVIEW) == 0;
+  if (postdriver)
+    commands[POST_INDEX].set_name(postdriver);
+  int gxditview_flag = postdriver && strcmp(xbasename(postdriver), GXDITVIEW) == 0;
   if (gxditview_flag && argc - optind == 1) {
     commands[POST_INDEX].append_arg("-title");
     commands[POST_INDEX].append_arg(argv[optind]);
@@ -307,7 +346,12 @@ int main(int argc, char **argv)
     commands[SPOOL_INDEX].set_name(0);
   }
   commands[TROFF_INDEX].append_arg("-T", device);
-  commands[EQN_INDEX].append_arg("-T", device);
+  // html renders equations as images via ps
+  if (strcmp(device, "html") == 0)
+    commands[EQN_INDEX].append_arg("-Tps:html");
+  else
+    commands[EQN_INDEX].append_arg("-T", device);
+
   commands[GRN_INDEX].append_arg("-T", device);
 
   int first_index;
@@ -335,11 +379,37 @@ int main(int argc, char **argv)
     if (putenv(strsave(e.contents())))
       fatal("putenv failed");
   }
+  {
+    // we save the original path in GROFF_PATH__ and put it into the
+    // environment -- troff will pick it up later.
+    char *path = getenv("PATH");
+    string e = "GROFF_PATH__";
+    e += '=';
+    if (path && *path)
+      e += path;
+    e += '\0';
+    if (putenv(strsave(e.contents())))
+      fatal("putenv failed");
+    char *binpath = getenv("GROFF_BIN_PATH");
+    string f = "PATH";
+    f += '=';
+    if (binpath && *binpath)
+      f += binpath;
+    else
+      f += BINPATH;
+    if (path && *path) {
+      f += PATH_SEP[0];
+      f += path;
+    }
+    f += '\0';
+    if (putenv(strsave(f.contents())))
+      fatal("putenv failed");
+  }
   if (Vflag) {
     print_commands();
     exit(0);
   }
-  return run_commands();
+  return run_commands(vflag);
 }
 
 const char *xbasename(const char *s)
@@ -372,6 +442,21 @@ void handle_unknown_desc_command(const char *command, const char *arg,
     else
       spooler = strsave(arg);
   }
+  if (strcmp(command, "prepro") == 0) {
+    if (arg == 0)
+      error_with_file_and_line(filename, lineno,
+			       "`prepro' command requires an argument");
+    else {
+      for (const char *p = arg; *p; p++)
+	if (csspace(*p)) {
+	  error_with_file_and_line(filename, lineno,
+				   "invalid `prepro' argument `%1'"
+				   ": program name required", arg);
+	  return;
+	}
+      predriver = strsave(arg);
+    }
+  }
   if (strcmp(command, "postpro") == 0) {
     if (arg == 0)
       error_with_file_and_line(filename, lineno,
@@ -384,7 +469,7 @@ void handle_unknown_desc_command(const char *command, const char *arg,
 				   ": program name required", arg);
 	  return;
 	}
-      driver = strsave(arg);
+      postdriver = strsave(arg);
     }
   }
 }
@@ -402,14 +487,14 @@ void print_commands()
 
 // Run the commands. Return the code with which to exit.
 
-int run_commands()
+int run_commands(int no_pipe)
 {
   char **v[NCOMMANDS];
   int j = 0;
   for (int i = 0; i < NCOMMANDS; i++)
     if (commands[i].get_name() != 0)
       v[j++] = commands[i].get_argv();
-  return run_pipeline(j, v);
+  return run_pipeline(j, v, no_pipe);
 }
 
 possible_command::possible_command()
@@ -569,9 +654,9 @@ char **possible_command::get_argv()
   return argv;
 }
 
-void synopsis()
+void synopsis(FILE *stream)
 {
-  fprintf(stderr,
+  fprintf(stream,
 "usage: %s [-abeghilpstvzCENRSUVXZ] [-Fdir] [-mname] [-Tdev] [-ffam]\n"
 "       [-wname] [-Wname] [-Mdir] [-dcs] [-rcn] [-nnum] [-olist] [-Parg]\n"
 "       [-Larg] [-Idir] [files...]\n",
@@ -580,7 +665,7 @@ void synopsis()
 
 void help()
 {
-  synopsis();
+  synopsis(stdout);
   fputs("\n"
 "-h\tprint this message\n"
 "-t\tpreprocess with tbl\n"
@@ -619,15 +704,14 @@ void help()
 "-U\tenable unsafe mode\n"
 "-Idir\tsearch dir for soelim.  Implies -s\n"
 "\n",
-	stderr);
+	stdout);
   exit(0);
 }
 
-void usage()
+void usage(FILE *stream)
 {
-  synopsis();
-  fprintf(stderr, "%s -h gives more help\n", program_name);
-  exit(1);
+  synopsis(stream);
+  fprintf(stream, "%s -h gives more help\n", program_name);
 }
 
 extern "C" {
