@@ -1,4 +1,4 @@
-/* Copyright (C) 1989, 1990, 1991, 1992 Free Software Foundation, Inc.
+/* Copyright (C) 1989, 1990, 1991, 1992, 2000 Free Software Foundation, Inc.
      Written by James Clark (jjc@jclark.com)
 
 This file is part of groff.
@@ -34,10 +34,6 @@ Compile options are:
 #include <unistd.h>
 #endif
 
-#ifndef errno
-extern int errno;
-#endif
-
 extern char *strerror();
 
 #ifdef _POSIX_VERSION
@@ -154,9 +150,8 @@ system_shell_dash_c (void)
 int
 is_system_shell (const char *shell)
 {
-  char monocased_shell[FILENAME_MAX];
   size_t shlen;
-  int ibase = 0, idot, i;
+  size_t ibase = 0, idot, i;
 
   if (!shell)	/* paranoia */
     return 0;
@@ -164,36 +159,30 @@ is_system_shell (const char *shell)
 
   for (i = 0; i < shlen; i++)
     {
-      if (isalpha(shell[i]))
-	monocased_shell[i] = tolower(shell[i]);
-      else
+      if (shell[i] == '.')
+	idot = i;
+      else if (shell[i] == '/' || shell[i] == '\\' || shell[i] == ':')
 	{
-	  monocased_shell[i] = shell[i];
-	  if (shell[i] == '.')
-	    idot = i;
-	  else if (shell[i] == '/' || shell[i] == '\\' || shell[i] == ':')
-	    {
-	      ibase = i + 1;
-	      idot = shlen;
-	    }
+	  ibase = i + 1;
+	  idot = shlen;
 	}
     }
-  monocased_shell[i] = '\0';
 
   /* "sh" and "sh.exe" should compare equal.  */
   return
-    strncmp(monocased_shell + ibase, system_shell_name(), idot - ibase) == 0
-    && (idot == shlen
-	|| strcmp(monocased_shell + idot, ".exe") == 0
-	|| strcmp(monocased_shell + idot, ".com") == 0);
+    (strncasecmp (shell + ibase, system_shell_name (), idot - ibase) == 0
+     && (idot == shlen
+	 || strcasecmp (shell + idot, ".exe") == 0
+	 || strcasecmp (shell + idot, ".com") == 0));
 }
 
 /* MSDOS doesn't have `fork', so we need to simulate the pipe by
    running the programs in sequence with redirected standard streams.  */
 
-int run_pipeline (ncommands, commands)
+int run_pipeline (ncommands, commands, no_pipe)
      int ncommands;
      char ***commands;
+     int no_pipe;
 {
   int save_stdin = dup(0);
   int save_stdout = dup(1);
@@ -222,7 +211,7 @@ int run_pipeline (ncommands, commands)
 	  if (close(f) < 0)
 	    sys_fatal("close stdin");
 	}
-      if (i < ncommands - 1)
+      if ((i < ncommands - 1) && !no_pipe)
 	{
 	  /* redirect stdout to temp file */
 	  f = open(tmpfiles[outfile], O_WRONLY|O_CREAT|O_TRUNC|O_BINARY, 0666);
@@ -277,9 +266,10 @@ int run_pipeline (ncommands, commands)
 
 #else  /* not __MSDOS__, not _WIN32 */
 
-int run_pipeline(ncommands, commands)
+int run_pipeline(ncommands, commands, no_pipe)
      int ncommands;
      char ***commands;
+     int no_pipe;
 {
   int i;
   int last_input = 0;
@@ -290,7 +280,7 @@ int run_pipeline(ncommands, commands)
   for (i = 0; i < ncommands; i++) {
       int pdes[2];
       PID_T pid;
-      if (i != ncommands - 1) {
+      if ((i != ncommands - 1) && !no_pipe) {
 	if (pipe(pdes) < 0)
 	  sys_fatal("pipe");
       }
@@ -307,7 +297,7 @@ int run_pipeline(ncommands, commands)
 	  if (close(last_input) < 0)
 	    sys_fatal("close");
 	}
-	if (i != ncommands - 1) {
+	if ((i != ncommands - 1) && !no_pipe) {
 	  if (close(1) < 0)
 	    sys_fatal("close");
 	  if (dup(pdes[1]) < 0)
@@ -328,7 +318,7 @@ int run_pipeline(ncommands, commands)
 	if (close(last_input) < 0)
 	  sys_fatal("close");
       }
-      if (i != ncommands - 1) {
+      if ((i != ncommands - 1) && !no_pipe) {
 	if (close(pdes[1]) < 0)
 	  sys_fatal("close");
 	last_input = pdes[0];
