@@ -1,5 +1,6 @@
 // -*- C++ -*-
-/* Copyright (C) 1989, 1990, 1991, 1992 Free Software Foundation, Inc.
+/* Copyright (C) 1989, 1990, 1991, 1992, 2000, 2001
+   Free Software Foundation, Inc.
      Written by James Clark (jjc@jclark.com)
 
 This file is part of groff.
@@ -88,12 +89,20 @@ top_level_diversion *topdiv;
 
 diversion *curdiv;
 
-void do_divert(int append)
+void do_divert(int append, int boxing)
 {
   tok.skip();
   symbol nm = get_name();
   if (nm.is_null()) {
     if (curdiv->prev) {
+      if (boxing) {
+	curenv->line = curdiv->saved_line;
+	curenv->width_total = curdiv->saved_width_total;
+	curenv->space_total = curdiv->saved_space_total;
+	curenv->saved_indent = curdiv->saved_saved_indent;
+	curenv->target_text_length = curdiv->saved_target_text_length;
+	curenv->prev_line_interrupted = curdiv->saved_prev_line_interrupted;
+      }
       diversion *temp = curdiv;
       curdiv = curdiv->prev;
       delete temp;
@@ -105,20 +114,40 @@ void do_divert(int append)
     macro_diversion *md = new macro_diversion(nm, append);
     md->prev = curdiv;
     curdiv = md;
+    if (boxing) {
+      curdiv->saved_line = curenv->line;
+      curdiv->saved_width_total = curenv->width_total;
+      curdiv->saved_space_total = curenv->space_total;
+      curdiv->saved_saved_indent = curenv->saved_indent;
+      curdiv->saved_target_text_length = curenv->target_text_length;
+      curdiv->saved_prev_line_interrupted = curenv->prev_line_interrupted;
+      curenv->line = 0;
+      curenv->start_line();
+    }
   }
   skip_line();
 }
 
 void divert()
 {
-  do_divert(0);
+  do_divert(0, 0);
 }
 
 void divert_append()
 {
-  do_divert(1);
+  do_divert(1, 0);
 }
   
+void box()
+{
+  do_divert(0, 1);
+}
+
+void box_append()
+{
+  do_divert(1, 1);
+}
+
 void diversion::need(vunits n)
 {
   vunits d = distance_to_next_trap();
@@ -328,7 +357,7 @@ vunits top_level_diversion::distance_to_next_trap()
 }
 
 void top_level_diversion::output(node *nd, int retain_size,
-				 vunits vs, vunits post_vs, hunits /*width*/)
+				 vunits vs, vunits post_vs, hunits width)
 {
   no_space_mode = 0;
   vunits next_trap_pos;
@@ -346,7 +375,7 @@ void top_level_diversion::output(node *nd, int retain_size,
   vertical_position += v.pre;
   vertical_position += v.pre_extra;
   the_output->print_line(page_offset, vertical_position, nd,
-			 v.pre + v.pre_extra, v.post_extra);
+			 v.pre + v.pre_extra, v.post_extra, width);
   vertical_position += v.post_extra;
   if (vertical_position > high_water_mark)
     high_water_mark = vertical_position;
@@ -610,6 +639,7 @@ void page_offset()
     n = topdiv->prev_page_offset;
   topdiv->prev_page_offset = topdiv->page_offset;
   topdiv->page_offset = n;
+  curenv->add_html_tag(".po", n.to_units());
   skip_line();
 }
 
@@ -730,6 +760,7 @@ void space_request()
   else
     // The line might have had line spacing that was truncated.
     truncated_space += n;
+  curenv->add_html_tag(".sp", n.to_units());
   tok.next();
 }
 
@@ -769,12 +800,12 @@ vunits saved_space;
 void save_vertical_space()
 {
   vunits x;
-  if (get_vunits(&x, 'v')) {
-    if (curdiv->distance_to_next_trap() > x)
-      curdiv->space(x, 1);
-    else
-      saved_space = x;
-  }
+  if (!has_arg() || !get_vunits(&x, 'v'))
+    x = curenv->get_vertical_spacing();
+  if (curdiv->distance_to_next_trap() > x)
+    curdiv->space(x, 1);
+  else
+    saved_space = x;
   skip_line();
 }
 
@@ -796,6 +827,7 @@ void flush_output()
     curenv->do_break();
   if (the_output)
     the_output->flush();
+  curenv->add_html_tag(".fl");
   tok.next();
 }
 
@@ -1093,6 +1125,8 @@ void init_div_requests()
   init_request("sp", space_request);
   init_request("di", divert);
   init_request("da", divert_append);
+  init_request("box", box);
+  init_request("boxa", box_append);
   init_request("bp", begin_page);
   init_request("ne", need_space);
   init_request("pn", page_number);
