@@ -1,5 +1,6 @@
 // -*- C++ -*-
-/* Copyright (C) 1989, 1990, 1991, 1992 Free Software Foundation, Inc.
+/* Copyright (C) 1989, 1990, 1991, 1992, 2000, 2001
+   Free Software Foundation, Inc.
      Written by James Clark (jjc@jclark.com)
 
 This file is part of groff.
@@ -29,7 +30,6 @@ Foundation, 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA. */
 #include "div.h"
 #include "reg.h"
 #include "charinfo.h"
-#include "searchpath.h"
 #include "macropath.h"
 #include <math.h>
 
@@ -133,9 +133,10 @@ void environment::output(node *nd, int no_fill, vunits vs, vunits post_vs,
 #ifdef WIDOW_CONTROL
       && (!widow_control || no_fill)
 #endif /* WIDOW_CONTROL */
-      )
+      ) {
     curdiv->output(nd, no_fill, vs, post_vs, width);
-  else {
+    emitted_node = 1;
+  } else {
     pending_output_line **p;
     for (p = &pending_lines; *p; p = &(*p)->next)
       ;
@@ -261,6 +262,7 @@ void leader_character()
 
 void environment::add_char(charinfo *ci)
 {
+  int s;
   if (interrupted)
     ;
   // don't allow fields in dummy environments
@@ -276,7 +278,7 @@ void environment::add_char(charinfo *ci)
     if (tab_contents == 0)
       tab_contents = new line_start_node;
     if (ci != hyphen_indicator_char)
-      tab_contents = tab_contents->add_char(ci, this, &tab_width);
+      tab_contents = tab_contents->add_char(ci, this, &tab_width, &s);
     else
       tab_contents = tab_contents->add_discretionary_hyphen();
   }
@@ -284,7 +286,7 @@ void environment::add_char(charinfo *ci)
     if (line == 0)
       start_line();
     if (ci != hyphen_indicator_char)
-      line = line->add_char(ci, this, &width_total);
+      line = line->add_char(ci, this, &width_total, &space_total);
     else
       line = line->add_discretionary_hyphen();
   }
@@ -376,21 +378,31 @@ void environment::space_newline()
   if (interrupted)
     return;
   hunits x = H0;
+  hunits sw = env_space_width(this);
+  hunits ssw = env_sentence_space_width(this);
   if (!translate_space_to_dummy) {
-    x = env_space_width(this);
+    x = sw;
     if (node_list_ends_sentence(line) == 1)
-      x += env_sentence_space_width(this);
+      x += ssw;
   }
-  if (line != 0 && line->merge_space(x)) {
+  width_list *w = new width_list(sw, ssw);
+  if (node_list_ends_sentence(line) == 1)
+    w->next = new width_list(sw, ssw);
+  if (line != 0 && line->merge_space(x, sw, ssw)) {
     width_total += x;
     return;
   }
-  add_node(new word_space_node(x));
-  possibly_break_line(spread_flag);
+  add_node(new word_space_node(x, w));
+  possibly_break_line(0, spread_flag);
   spread_flag = 0;
 }
 
 void environment::space()
+{
+  space(env_space_width(this), env_sentence_space_width(this));
+}
+
+void environment::space(hunits space_width, hunits sentence_space_width)
 {
   if (interrupted)
     return;
@@ -398,26 +410,30 @@ void environment::space()
     add_padding();
     return;
   }
-  hunits x = translate_space_to_dummy ? H0 : env_space_width(this);
+  hunits x = translate_space_to_dummy ? H0 : space_width;
   node *p = current_tab ? tab_contents : line;
   hunits *tp = current_tab ? &tab_width : &width_total;
   if (p && p->nspaces() == 1 && p->width() == x
       && node_list_ends_sentence(p->next) == 1) {
-    hunits xx = translate_space_to_dummy ? H0 : env_sentence_space_width(this);
-    if (p->merge_space(xx)) {
+    hunits xx = translate_space_to_dummy ? H0 : sentence_space_width;
+    if (p->merge_space(xx, space_width, sentence_space_width)) {
       *tp += xx;
       return;
     }
   }
-  if (p && p->merge_space(x)) {
+  if (p && p->merge_space(x, space_width, sentence_space_width)) {
     *tp += x;
     return;
   }
-  add_node(new word_space_node(x));
-  possibly_break_line(spread_flag);
+  add_node(new word_space_node(x,
+			       new width_list(space_width,
+					      sentence_space_width)));
+  possibly_break_line(0, spread_flag);
   spread_flag = 0;
 }
 
+node *do_underline_special(int);
+
 void environment::set_font(symbol nm)
 {
   if (interrupted)
@@ -441,6 +457,12 @@ void environment::set_font(symbol nm)
       return;
     fontno = n;
   }
+  if (underline_spaces && fontno != prev_fontno) {
+    if (fontno == get_underline_fontno())
+      add_node(do_underline_special(1));
+    if (prev_fontno == get_underline_fontno())
+      add_node(do_underline_special(0));
+  }
 }
 
 void environment::set_font(int n)
@@ -541,6 +563,7 @@ environment::environment(symbol nm)
   temporary_indent(0),
   have_temporary_indent(0),
   underline_lines(0),
+  underline_spaces(0),
   input_trap_count(0),
   line(0),
   prev_text_length(0),
@@ -548,6 +571,7 @@ environment::environment(symbol nm)
   space_total(0),
   input_line_start(0),
   tabs(units_per_inch/2, TAB_LEFT),
+  line_tabs(0),
   current_tab(TAB_NONE),
   leader_node(0),
   tab_char(0),
@@ -573,6 +597,9 @@ environment::environment(symbol nm)
 #ifdef WIDOW_CONTROL
   widow_control(0),
 #endif /* WIDOW_CONTROL */
+  need_eol(0),
+  ignore_next_eol(0),
+  emitted_node(0),
   name(nm),
   control_char('.'),
   no_break_control_char('\''),
@@ -622,6 +649,7 @@ environment::environment(const environment *e)
   temporary_indent(0),
   have_temporary_indent(0),
   underline_lines(0),
+  underline_spaces(0),
   input_trap_count(0),
   line(0),
   prev_text_length(e->prev_text_length),
@@ -629,6 +657,7 @@ environment::environment(const environment *e)
   space_total(0),
   input_line_start(0),
   tabs(e->tabs),
+  line_tabs(e->line_tabs),
   current_tab(TAB_NONE),
   leader_node(0),
   tab_char(e->tab_char),
@@ -654,6 +683,8 @@ environment::environment(const environment *e)
 #ifdef WIDOW_CONTROL
   widow_control(e->widow_control),
 #endif /* WIDOW_CONTROL */
+  need_eol(0),
+  ignore_next_eol(0),
   name(e->name),		// so that eg `.if "\n[.ev]"0"' works
   control_char(e->control_char),
   no_break_control_char(e->no_break_control_char),
@@ -692,6 +723,7 @@ void environment::copy(const environment *e)
   have_temporary_indent = 0;
   temporary_indent = 0;
   underline_lines = 0;
+  underline_spaces = 0;
   input_trap_count = 0;
   prev_text_length = e->prev_text_length;
   width_total = 0;
@@ -705,6 +737,7 @@ void environment::copy(const environment *e)
   pending_lines = 0;
   discarding = 0;
   tabs = e->tabs;
+  line_tabs = e->line_tabs;
   current_tab = TAB_NONE;
   current_field = 0;
   margin_character_flags = e->margin_character_flags;
@@ -1062,9 +1095,8 @@ void font_change()
 
 void family_change()
 {
-  symbol s = get_name(1);
-  if (!s.is_null())
-    curenv->set_family(s);
+  symbol s = get_name();
+  curenv->set_family(s);
   skip_line();
 }
 
@@ -1075,6 +1107,7 @@ void point_size()
     if (n <= 0)
       n = 1;
     curenv->set_size(n);
+    curenv->add_html_tag(".ps", n);
   }
   else
     curenv->set_size(0);
@@ -1084,7 +1117,7 @@ void point_size()
 void space_size()
 {
   int n;
-  if (get_integer(&n)) {
+  if (get_integer(&n) && !compatible_flag) {
     curenv->space_size = n;
     if (has_arg() && get_integer(&n))
       curenv->sentence_space_size = n;
@@ -1101,6 +1134,7 @@ void fill()
   if (break_flag)
     curenv->do_break();
   curenv->fill = 1;
+  curenv->add_html_tag(".fi");
   tok.next();
 }
 
@@ -1110,7 +1144,11 @@ void no_fill()
     tok.next();
   if (break_flag)
     curenv->do_break();
+
   curenv->fill = 0;
+  curenv->add_html_tag(".nf");
+  curenv->ignore_next_eol = 1;
+  curenv->add_html_tag(".po", topdiv->get_page_offset().to_units());
   tok.next();
 }
 
@@ -1127,6 +1165,7 @@ void center()
     curenv->do_break();
   curenv->right_justify_lines = 0;
   curenv->center_lines = n;
+  curenv->add_html_tag(".ce", n);
   tok.next();
 }
 
@@ -1143,6 +1182,7 @@ void right_justify()
     curenv->do_break();
   curenv->center_lines = 0;
   curenv->right_justify_lines = n;
+  curenv->add_html_tag(".rj", n);
   tok.next();
 }
 
@@ -1159,6 +1199,7 @@ void line_length()
     temp = curenv->prev_line_length;
   curenv->prev_line_length = curenv->line_length;
   curenv->line_length = temp;
+  curenv->add_html_tag(".ll", temp.to_units());
   skip_line();
 }
 
@@ -1245,6 +1286,7 @@ void indent()
   curenv->have_temporary_indent = 0;
   curenv->prev_indent = curenv->indent;
   curenv->indent = temp;
+  curenv->add_html_tag(".in", temp.to_units());
   tok.next();
 }
 
@@ -1265,11 +1307,20 @@ void temporary_indent()
   if (!err) {
     curenv->temporary_indent = temp;
     curenv->have_temporary_indent = 1;
+    curenv->add_html_tag(".ti", temp.to_units());
   }
   tok.next();
 }
 
-void underline()
+node *do_underline_special(int underline_spaces)
+{
+  macro m;
+  m.append_str("x u ");
+  m.append(underline_spaces + '0');
+  return new special_node(m, 1);
+}
+
+void do_underline(int underline_spaces)
 {
   int n;
   if (!has_arg() || !get_integer(&n))
@@ -1278,6 +1329,10 @@ void underline()
     if (curenv->underline_lines > 0) {
       curenv->prev_fontno = curenv->fontno;
       curenv->fontno = curenv->pre_underline_fontno;
+      if (underline_spaces) {
+	curenv->underline_spaces = 0;
+	curenv->add_node(do_underline_special(0));
+      }
     }
     curenv->underline_lines = 0;
   }
@@ -1285,10 +1340,24 @@ void underline()
     curenv->underline_lines = n;
     curenv->pre_underline_fontno = curenv->fontno;
     curenv->fontno = get_underline_fontno();
+    if (underline_spaces) {
+      curenv->underline_spaces = 1;
+      curenv->add_node(do_underline_special(1));
+    }
   }
   skip_line();
 }
 
+void continuous_underline()
+{
+  do_underline(1);
+}
+
+void underline()
+{
+  do_underline(0);
+}
+
 void control_char()
 {
   curenv->control_char = '.';
@@ -1459,6 +1528,10 @@ void environment::newline()
     if (--underline_lines == 0) {
       prev_fontno = fontno;
       fontno = pre_underline_fontno;
+      if (underline_spaces) {
+        underline_spaces = 0;
+        add_node(do_underline_special(0));
+      }
     }
   }
   if (current_field)
@@ -1722,18 +1795,21 @@ breakpoint *environment::choose_breakpoint()
   return 0;
 }
 
-void environment::hyphenate_line()
+void environment::hyphenate_line(int start_here)
 {
   if (line == 0)
     return;
   hyphenation_type prev_type = line->get_hyphenation_type();
   node **startp;
-  for (startp = &line->next; *startp != 0; startp = &(*startp)->next) {
-    hyphenation_type this_type = (*startp)->get_hyphenation_type();
-    if (prev_type == HYPHEN_BOUNDARY && this_type == HYPHEN_MIDDLE)
-      break;
-    prev_type = this_type;
-  }
+  if (start_here)
+    startp = &line;
+  else
+    for (startp = &line->next; *startp != 0; startp = &(*startp)->next) {
+      hyphenation_type this_type = (*startp)->get_hyphenation_type();
+      if (prev_type == HYPHEN_BOUNDARY && this_type == HYPHEN_MIDDLE)
+	break;
+      prev_type = this_type;
+    }
   if (*startp == 0)
     return;
   node *tem = *startp;
@@ -1808,7 +1884,7 @@ static void distribute_space(node *n, int nspaces, hunits desired_space,
   assert(desired_space.is_zero() && nspaces == 0);
 }
 
-void environment::possibly_break_line(int forced)
+void environment::possibly_break_line(int start_here, int forced)
 {
   if (!fill || current_tab || current_field || dummy)
     return;
@@ -1817,7 +1893,7 @@ void environment::possibly_break_line(int forced)
 	     // When a macro follows a paragraph in fill mode, the
 	     // current line should not be empty.
 	     || (width_total - line->width()) > target_text_length)) {
-    hyphenate_line();
+    hyphenate_line(start_here);
     breakpoint *bp = choose_breakpoint();
     if (bp == 0)
       // we'll find one eventually
@@ -1936,6 +2012,129 @@ void environment::final_break()
     do_break();
 }
 
+/*
+ *  add_html_tag_eol - add an end of line tag if appropriate.
+ */
+
+void environment::add_html_tag_eol()
+{
+  if (is_html) {
+    if (ignore_next_eol > 0)
+      ignore_next_eol--;
+    else if (need_eol > 0) {
+      need_eol--;
+      add_html_tag("eol");
+    }
+    else if (!fill && emitted_node) {
+      add_html_tag("eol");
+      emitted_node = 0;
+    }
+  }
+}
+
+/*
+ *  add_html_tag - emits a special html-tag: to help post-grohtml understand
+ *                 the key troff commands
+ */
+
+void environment::add_html_tag(const char *name)
+{
+  if (is_html) {
+    /*
+     * need to emit tag for post-grohtml
+     * but we check to see whether we can emit specials
+     */
+    if (curdiv == topdiv && topdiv->before_first_page)
+      topdiv->begin_page();
+    macro *m = new macro;
+    m->append_str("html-tag:");
+    for (const char *p = name; *p; p++)
+      if (!illegal_input_char((unsigned char)*p))
+	m->append(*p);
+    add_node(new special_node(*m));
+  }
+}
+
+/*
+ *  add_html_tag - emits a special html-tag: to help post-grohtml understand
+ *                 the key troff commands, it appends a string representation
+ *                 of i.
+ */
+
+void environment::add_html_tag(const char *name, int i)
+{
+  if (is_html) {
+    if (strcmp(name, ".ce") == 0) {
+      if (i == 0)
+	need_eol = 0;
+      else {
+	need_eol = i;
+	ignore_next_eol = 1;  // since the .ce creates an eol
+      }
+    }
+
+    /*
+     * need to emit tag for post-grohtml
+     * but we check to see whether we can emit specials
+     */
+    if (curdiv == topdiv && topdiv->before_first_page)
+      topdiv->begin_page();
+    macro *m = new macro;
+    m->append_str("html-tag:");
+    for (const char *p = name; *p; p++)
+      if (!illegal_input_char((unsigned char)*p))
+	m->append(*p);
+    m->append(' ');
+    m->append_int(i);
+    // output_pending_lines();
+    output(new special_node(*m), !fill, 0, 0, 0);
+    // output_pending_lines();
+  }
+}
+
+/*
+ *  add_html_tag_tabs - emits the tab settings for post-grohtml
+ */
+
+void environment::add_html_tag_tabs()
+{
+  if (is_html) {
+    /*
+     * need to emit tag for post-grohtml
+     * but we check to see whether we can emit specials
+     */
+    if (curdiv == topdiv && topdiv->before_first_page)
+      topdiv->begin_page();
+    macro *m = new macro;
+    hunits d, l;
+    enum tab_type t;
+    m->append_str("html-tag:.ta ");  
+    do {
+      t = curenv->tabs.distance_to_next_tab(l, &d);
+      l += d;
+      switch (t) {
+      case TAB_LEFT:
+	m->append_str(" L ");
+	m->append_int(d.to_units());
+	break;
+      case TAB_CENTER:
+	m->append_str(" C ");
+	m->append_int(d.to_units());
+	break;
+      case TAB_RIGHT:
+	m->append_str(" R ");
+	m->append_int(d.to_units());
+	break;
+      case TAB_NONE:
+	break;
+      }
+    } while ((t != TAB_NONE) && (l < get_line_length()));
+    output_pending_lines();
+    output(new special_node(*m), !fill, 0, 0, 0);
+    output_pending_lines();
+  }
+}
+
 void environment::do_break()
 {
   if (curdiv == topdiv && topdiv->before_first_page) {
@@ -1990,8 +2189,10 @@ void break_request()
 {
   while (!tok.newline() && !tok.eof())
     tok.next();
-  if (break_flag)
+  if (break_flag) {
     curenv->do_break();
+    curenv->add_html_tag(".br");
+  }
   tok.next();
 }
 
@@ -2362,6 +2563,7 @@ void set_tabs()
     }
   }
   curenv->tabs = tabs;
+  curenv->add_html_tag_tabs();
   skip_line();
 }
 
@@ -2388,7 +2590,9 @@ void tabs_restore()
 
 tab_type environment::distance_to_next_tab(hunits *distance)
 {
-  return curenv->tabs.distance_to_next_tab(get_input_line_position(), distance);
+  return line_tabs
+    ? curenv->tabs.distance_to_next_tab(get_text_length(), distance)
+    : curenv->tabs.distance_to_next_tab(get_input_line_position(), distance);
 }
 
 void field_characters()
@@ -2401,6 +2605,21 @@ void field_characters()
   skip_line();
 }
 
+void line_tabs_request()
+{
+  int n;
+  if (has_arg() && get_integer(&n))
+    curenv->line_tabs = n != 0;
+  else
+    curenv->line_tabs = 1;
+  skip_line();
+}
+
+int environment::get_line_tabs()
+{
+  return line_tabs;
+}
+
 void environment::wrap_up_tab()
 {
   if (!current_tab)
@@ -2441,7 +2660,7 @@ void environment::wrap_up_tab()
   }
   tab_field_spaces = 0;
   tab_contents = 0;
-  tab_width =  H0;
+  tab_width = H0;
   tab_distance = H0;
   current_tab = TAB_NONE;
 }
@@ -2454,7 +2673,7 @@ node *environment::make_tab_node(hunits d, node *next)
     leader_node = 0;
   }
   if (!leader_node)
-    return new hmotion_node(d, next);
+    return new hmotion_node(d, 1, 0, next);
   node *n = new hline_node(d, leader_node, next);
   leader_node = 0;
   return n;
@@ -2473,10 +2692,12 @@ void environment::handle_tab(int is_leader)
   case TAB_NONE:
     return;
   case TAB_LEFT:
+    add_html_tag("tab left");
     add_node(make_tab_node(d));
     return;
   case TAB_RIGHT:
   case TAB_CENTER:
+    add_html_tag("tab center");
     tab_width = 0;
     tab_distance = d;
     tab_contents = 0;
@@ -2789,12 +3010,13 @@ void init_env_requests()
   init_request("in", indent);
   init_request("ti", temporary_indent);
   init_request("ul", underline);
-  init_request("cu", underline);
+  init_request("cu", continuous_underline);
   init_request("cc", control_char);
   init_request("c2", no_break_control_char);
   init_request("br", break_request);
   init_request("tl", title);
   init_request("ta", set_tabs);
+  init_request("linetabs", line_tabs_request);
   init_request("fc", field_characters);
   init_request("mc", margin_character);
   init_request("nn", no_number);
@@ -2819,6 +3041,7 @@ void init_env_requests()
   init_int_env_reg(".b", get_bold);
   init_hunits_env_reg(".i", get_indent);
   init_hunits_env_reg(".in", get_saved_indent);
+  init_int_env_reg(".int", get_prev_line_interrupted);
   init_int_env_reg(".j", get_adjust_mode);
   init_hunits_env_reg(".k", get_text_length);
   init_hunits_env_reg(".l", get_line_length);
@@ -2842,6 +3065,7 @@ void init_env_requests()
   init_int_env_reg(".hlc", get_hyphen_line_count);
   init_hunits_env_reg(".lt", get_title_length);
   init_string_env_reg(".tabs", get_tabs);
+  init_int_env_reg(".linetabs", get_line_tabs);
   init_hunits_env_reg(".csk", get_prev_char_skew);
   init_vunits_env_reg(".cht", get_prev_char_height);
   init_vunits_env_reg(".cdp", get_prev_char_depth);
@@ -3058,7 +3282,7 @@ void hyphen_trie::insert_pattern(const char *pat, int patlen, int *num)
 void hyphen_trie::hyphenate(const char *word, int len, int *hyphens)
 {
   int j;
-  for (j = 0; j < len+1; j++)
+  for (j = 0; j < len + 1; j++)
     hyphens[j] = 0;
   for (j = 0; j < len - 1; j++) {
     h = hyphens + j;
@@ -3097,7 +3321,7 @@ void hyphen_trie::read_patterns_file(const char *name)
   int num[WORD_MAX+1];
   errno = 0;
   char *path = 0;
-  FILE *fp = macro_path.open_file(name, &path);
+  FILE *fp = mac_path->open_file(name, &path);
   if (fp == 0) {
     error("can't find hyphenation patterns file `%1'", name);
     return;
