@@ -1,5 +1,6 @@
 // -*- C++ -*-
-/* Copyright (C) 1989, 1990, 1991, 1992 Free Software Foundation, Inc.
+/* Copyright (C) 1989, 1990, 1991, 1992, 2000, 2001
+   Free Software Foundation, Inc.
      Written by James Clark (jjc@jclark.com)
 
 This file is part of groff.
@@ -33,7 +34,7 @@ class token {
     TOKEN_BACKSPACE,
     TOKEN_BEGIN_TRAP,
     TOKEN_CHAR,			// a normal printing character
-    TOKEN_DUMMY,
+    TOKEN_DUMMY,		// \&
     TOKEN_EMPTY,		// this is the initial value
     TOKEN_END_TRAP,
     TOKEN_ESCAPE,		// \e
@@ -52,8 +53,10 @@ class token {
     TOKEN_SPACE,		// ` ' -- ordinary space
     TOKEN_SPECIAL,		// a special character -- \' \` \- \(xx
     TOKEN_SPREAD,		// \p -- break and spread output line 
+    TOKEN_STRETCHABLE_SPACE,	// \~
     TOKEN_TAB,			// tab
     TOKEN_TRANSPARENT,		// \!
+    TOKEN_TRANSPARENT_DUMMY,	// \)
     TOKEN_EOF			// end of file
     } type;
 public:
@@ -66,7 +69,8 @@ public:
   void skip();
   int eof();
   int nspaces();		// 1 if space, 2 if double space, 0 otherwise
-  int space();			// is it a space or double space?
+  int space();			// is the current token a space?
+  int stretchable_space();	// is the current token a stretchable space?
   int white_space();		// is the current token space or tab?
   int special();                // is the current token a special character?
   int newline();		// is the current token a newline?
@@ -75,6 +79,7 @@ public:
   int backspace();
   int delimiter(int warn = 0);	// is it suitable for use as a delimiter?
   int dummy();
+  int transparent_dummy();
   int transparent();
   int left_brace();
   int right_brace();
@@ -105,6 +110,8 @@ extern void handle_initial_title();
 struct hunits;
 extern void read_title_parts(node **part, hunits *part_width);
 
+extern int get_number_rigidly(units *result, unsigned char si);
+
 extern int get_number(units *result, unsigned char si);
 extern int get_integer(int *result);
 
@@ -125,6 +132,11 @@ inline int token::space()
   return type == TOKEN_SPACE;
 }
 
+inline int token::stretchable_space()
+{
+  return type == TOKEN_STRETCHABLE_SPACE;
+}
+
 inline int token::special()
 { 
   return type == TOKEN_SPECIAL;
@@ -168,6 +180,11 @@ inline int token::dummy()
   return type == TOKEN_DUMMY;
 }
 
+inline int token::transparent_dummy()
+{
+  return type == TOKEN_TRANSPARENT_DUMMY;
+}
+
 inline int token::left_brace()
 {
   return type == TOKEN_LEFT_BRACE;
